# -*- coding: utf-8 -*-

from odoo import fields, models
from odoo.exceptions import UserError

from markupsafe import Markup


class PurchaseOrder(models.Model):
    _inherit = 'purchase.order'

    purchase_type = fields.Selection(selection_add=[('apps', 'Apps Commission')])

    def _merge_to_po(self, dest_po):
        """
        This function is used as an helper to merge two open purchase orders associated to a same vendor.
        The merging operation mainly consist in to update the 'price_unit' on the purchase order lines and
        to change their associated purchase order.

        params: dest_po record: it's the purchase order we want to keep at the end.
        """
        self.ensure_one()

        if self.state != 'draft' or dest_po.state != 'draft':
            raise UserError("Only open purchase orders can be merged together.")
        if self.partner_id != dest_po.partner_id:
            raise UserError("Only purchase orders associated to the same vendor can be merged together.")

        src_lines = self.order_line
        fpos = dest_po.fiscal_position_id

        # if the purchase orders do not use the same currency, we convert the lines
        for line in src_lines:
            vals = {'order_id': dest_po.id}
            taxes = line.product_id.supplier_taxes_id
            taxes_id = fpos.map_tax(taxes) if fpos else taxes
            if taxes_id:
                taxes_id = taxes_id.filtered(lambda x: x.company_id.id == dest_po.company_id.id)
            if line.taxes_id != taxes_id:
                vals['taxes_id'] = [(6, 0, taxes_id.ids)]
            # Convert to currency if the currency changes between two purchase orders
            # Otherwise, still write same price_unit, because writing the order_id alone
            # would otherwise recompute the price_unit with the current seller information
            # but we want to keep the exact same amounts for consistency
            vals['price_unit'] = self.currency_id._convert(line.price_unit, dest_po.currency_id,
                                                           dest_po.company_id, line.create_date)
            line.write(vals)

        self.button_cancel()
        # cancel the source PO and post a message on the chatter
        self.message_post(
            body=Markup("""This purchase order has been merged to <a href="#" data-oe-model="%s" data-oe-id="%d">%s</a>""") % (
                dest_po._name,
                dest_po.id,
                dest_po.name,
            )
        )
        # post a message on the purchase order destination
        dest_po.message_post(
            body=Markup("""This purchase order has been merged with <a href="#" data-oe-model="%s" data-oe-id="%d">%s</a>""") % (
                self._name,
                self.id,
                self.name,
            )
        )


class PurchaseOrderLine(models.Model):
    _inherit = 'purchase.order.line'

    sale_order_company_id = fields.Many2one(related='sale_order_id.company_id', store=True, string="Sale Order Company")
